package org.ilyin.fileSystemAPI;

import org.ilyin.model.IModelFiller;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.List;

/**
 * Implements a set of methods for filling file system model.
 * Uses information about real file system.
 * 
 * @author Alexander Ilyin
 */
public class RealFiller implements IModelFiller {
	public void getChildren(String path, List<String> dirs, List<String> files) {
		File file = new File(path);
		
		if (!file.exists())
			throw new RuntimeException("File(directory) " + path + " does not exist.");
		if (!file.isDirectory())
			throw new RuntimeException(path + ": not a directory!");
		
		File[] childrenFiles = file.listFiles();
		
		for(File f : childrenFiles) {
			//System.out.println(f.getName());
			if (f.isDirectory()) {
				dirs.add(f.getName());
			} else if (f.isFile()) {
				files.add(f.getName());
			} else {
				System.out.println("Not a file and not a dir: " + path);
				//throw new RuntimeException(f.getName() + ": not a dir and not a file");
			}
		}
	}
	
	public String getParent(String path) {
		File file = new File(path);

		if (!file.exists())
			throw new RuntimeException("File(directory) " + path + " does not exist.");
		
		File parent = file.getParentFile();
		if (parent == null)
			return null;

		try {
			return parent.getCanonicalPath();
		} catch(IOException e) {
			throw new RuntimeException("Can't get canonical path of the parent of " + path);
		}
	}

	public String getCanonicalPath(String path) {
		try {
			return (new File(path)).getCanonicalPath();
		} catch(IOException e) {
			throw new RuntimeException("Illegal path: " + path);
		}
	}
	
	public char separatorChar() {
		return File.separatorChar;
	}

	public String readChars(String path, int n) {
		if (n < 0)
			throw new IllegalArgumentException("Trying ro read " + n + 
											   " characters from " + path);
		
		try {
			char[] buf = new char[n];
			FileReader reader = new FileReader(path);
			int chRead = reader.read(buf, 0, n);
			if (chRead == -1)
				return "";
			if (chRead < n)
				return (new String(buf)).substring(0, chRead);
			
			return new String(buf);
		} catch(IOException e) {
			//System.out.println("Can't read file " + path);
			return null;
		}
	}

	public boolean rename(String path, String name) {
		File file = new File(path);
		
		String newPath = path.substring(0, path.lastIndexOf(separatorChar()) + 1);
		newPath += name;
		return file.renameTo(new File(newPath));
	}
	
	public boolean remove(String path) {
		File file = new File(path);
		
		return file.delete();
	}
	
	public boolean createFile(String path) {
		File file = new File(path);
		
		try {
			return file.createNewFile();
		} catch (IOException e) {
			throw new RuntimeException("Can't create file " + path);
		}
	}
	
	public boolean createDir(String path) {
		File file = new File(path);
		return file.mkdir();
	}
}
