package org.ilyin.gui;

import javax.swing.*;

import java.awt.event.*;

import org.ilyin.model.IFileManager;
import org.ilyin.settings.Settings;
import org.ilyin.gui.nortonView.NortonView;
import org.ilyin.gui.explorerView.ExplorerView;

public class MainWindow extends JFrame {
	private final static String WIDTH = "width";
	private final static String HEIGHT = "height";
	private final static String VIEW_NAME = "viewName";
	private final static String X_LOCATION = "x";
	private final static String Y_LOCATION = "y";
	
	private Settings mySettings;
	private IFileManager myManager;
	private AbstractView myView;
	//private JToolBar myToolBar;
	
	public MainWindow(IFileManager manager) {
		super("File Manager");
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		
		myManager = manager;

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {
			System.out.println("Look & Feel Exception");
			System.out.println(e.getMessage());
		}
		
		setJMenuBar(MenuFactory.createMainMenu(this));
		//myToolBar = ToolBarFactory.createToolBar();
		//myToolBar.setFloatable(false);
		
		addWindowListener(new WindowListener());
		addKeyListener(new MyKeyListener());
		
		mySettings = Settings.getInstance();
		
		// setting window location
		String x = mySettings.getProperty(X_LOCATION);
		String y = mySettings.getProperty(Y_LOCATION);
		//if ((x != null) && (y != null)) {
		try {
			setLocation(Integer.parseInt(x), Integer.parseInt(y));
		//} else {
		} catch (RuntimeException e) {
			setLocation(200, 100);
		}
		
		// setting width and height of the window
		String w = mySettings.getProperty(WIDTH);
		String h = mySettings.getProperty(HEIGHT);
		//if ((w != null) && (h != null)) {
		try {
			setSize(Integer.parseInt(w), Integer.parseInt(h));
		//} else {
		} catch (RuntimeException e) {
			setSize(400, 300);
		}

		// setting view
		String viewName = mySettings.getProperty(VIEW_NAME);
		if ((viewName == null) || (viewName.equals("norton style view"))) {// ?!
			myView = new NortonView(myManager);
			setContentPane(myView);
		} else {
			myView = new ExplorerView(myManager);
			setContentPane(((ExplorerView)myView).scrollPane());
		}
		//getContentPane().add(new JScrollPane(myView));
		
		//getContentPane().add(myToolBar);
		
		setVisible(true);
	}
	
	void setView(AbstractView view) {
		myView = view;
		if (view.getViewName().equals("explorer style view")) {
			setContentPane(((ExplorerView)myView).scrollPane());
		} else {
			setContentPane(myView);
		}
		//getContentPane().add(myToolBar);
	}
	
	private class WindowListener extends WindowAdapter {
		public void windowClosing(WindowEvent e) {
			myView.closing();
			
			mySettings.setProperty(X_LOCATION, ((Integer)MainWindow.this.getLocation().x).toString());
			mySettings.setProperty(Y_LOCATION, ((Integer)MainWindow.this.getLocation().y).toString());
			mySettings.setProperty(WIDTH, ((Integer)MainWindow.this.getWidth()).toString());
			mySettings.setProperty(HEIGHT, ((Integer)MainWindow.this.getHeight()).toString());
			mySettings.setProperty(VIEW_NAME, myView.getViewName());
			
			mySettings.save();
		}
	}

	// test listener
	private class MyKeyListener implements KeyListener {

		public void keyTyped(KeyEvent e) { }

		public void keyPressed(KeyEvent e) {
			if (e.getKeyCode() == KeyEvent.VK_LEFT) {
				//System.out.println("left");
				myView.left();
			} else if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
				//System.out.println("right");
				myView.right();
			} else if (e.getKeyCode() == KeyEvent.VK_ENTER) {
				myView.enter();
			}
		}

		public void keyReleased(KeyEvent e) { }
	}
	
	private static final long serialVersionUID = 5599580904763847890L;
}
