package org.ilyin.gui;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import org.ilyin.gui.explorerView.ExplorerView;
import org.ilyin.gui.nortonView.NortonView;
import org.ilyin.model.FileManagerHolder;

class MenuFactory {
	private MenuFactory() {
	}
	
	/**
	 * Returns menubar with all menus attached to it.
	 * 
	 * @return menubar
	 */
	public static JMenuBar createMainMenu(MainWindow w) {
		JMenuBar mainMenu = new JMenuBar();
		
		mainMenu.add(createFileMenu());
		mainMenu.add(createViewMenu(w));
		mainMenu.add(createPreferencesMenu(w));
		
		return mainMenu;
	}
	
	private static JMenu createFileMenu() {
		JMenu fileMenu = new JMenu("File");
		
		AbstractAction exitAction = 
			new AbstractAction() {
				public void actionPerformed(ActionEvent e) {
					//System.out.println("exit clicked");
					System.exit(0);
				}
				
				private static final long serialVersionUID = -201342711415936008L;
			};
		JMenuItem exit = new JMenuItem(exitAction);
		exit.setText("Exit");

		fileMenu.add(exit);
		
		return fileMenu;
	}
	
	private static JMenu createViewMenu(final MainWindow w) {
		JMenu viewMenu = new JMenu("View");
		
		// explorer style
		AbstractAction explorerAction = 
			new AbstractAction() {
				public void actionPerformed(ActionEvent e) {
					//w.setContentPane(new View2(FileManagerFactory.getFileManager()));
					w.setView(new ExplorerView(FileManagerHolder.getFileManager()));
					w.setVisible(true);
				}

				private static final long serialVersionUID = -8857230336750033028L;
			};
		JMenuItem explorerStyle = new JMenuItem(explorerAction);
		explorerStyle.setText("Explorer style");
		viewMenu.add(explorerStyle);

		AbstractAction nortonAction = 
			new AbstractAction() {
				public void actionPerformed(ActionEvent e) {
					//w.setContentPane(new View1(FileManagerFactory.getFileManager()));
					w.setView(new NortonView(FileManagerHolder.getFileManager()));
					w.setVisible(true);
				}

				private static final long serialVersionUID = 6627696205187007659L;
			};
		JMenuItem nortonStyle = new JMenuItem(nortonAction);
		nortonStyle.setText("Norton style");
		viewMenu.add(nortonStyle);
		
		return viewMenu;
	}
	
	private static JMenu createPreferencesMenu(final MainWindow w) {
		JMenu operationMenu = new JMenu("Preferences");
		
		AbstractAction programsAction = 
			new AbstractAction() {
				public void actionPerformed(ActionEvent e) {
					new ProgramsDialog(w);
				}
				
				private static final long serialVersionUID = -4573027861361662694L;
			};
		JMenuItem programs = new JMenuItem(programsAction);
		programs.setText("External programs");
		
		operationMenu.add(programs);
		
		return operationMenu;
	}
}
