package org.ilyin.gui;

import javax.swing.*;
import org.ilyin.settings.Settings;
import org.ilyin.model.*;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.IOException;
import java.util.List;
import java.util.Vector;

class ProgramsDialog extends JDialog {
	private final JComboBox myTypes;
	private final JTextField myCommand;
	private final JButton myOK, myBrowse;
	private final JLabel myExplanation;
	
	public ProgramsDialog(JFrame owner) {
		super(owner, true);
		
		List<IFileType> types = FileManagerHolder.getFileManager().getTypes();
		Vector<String> typeNames = new Vector<String>();
		for (IFileType type : types) {
			if ((!typeNames.contains(type.getName())) &&
				(!type.getName().equals("Directory"))) {	// add javadoc!
				typeNames.add(type.getName());
			}
		}
		myTypes = new JComboBox(typeNames/*types.toArray()*/);
		myTypes.setPrototypeDisplayValue("very very very very long name");
		//myTypes.setSize(220, myTypes.getHeight());
		myTypes.addItemListener(new TypeListener());
		
		myCommand = new JTextField(20);
		myCommand.setSize(myTypes.getWidth(), myCommand.getHeight());	// doesn't work!
		myCommand.setText(Settings.getInstance().getProperty(typeNames.firstElement()));
		
		AbstractAction browseAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser chooser = new JFileChooser();
				if (chooser.showOpenDialog(ProgramsDialog.this) == JFileChooser.APPROVE_OPTION) {
					try {
						myCommand.grabFocus();
						myCommand.setText(chooser.getSelectedFile().getCanonicalPath());
					} catch (IOException ex) {
					}
				}
			}

			private static final long serialVersionUID = -1859545384972441081L;
		};
		myBrowse = new JButton(browseAction);
		myBrowse.setText("Browse");
		
		myExplanation = new JLabel("<html>You can use \"&lt;filename&gt;\" in the command.<br>E.g.:  program.exe &lt;filename&gt;");

		AbstractAction okAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				String com = myCommand.getText();
				String type = (String)myTypes.getSelectedItem();
				Settings.getInstance().setProperty(type, com);
				ProgramsDialog.this.dispose();
			}
			
			private static final long serialVersionUID = -1859545384972441081L;
		};
		myOK = new JButton(okAction);
		myOK.setText("OK");

		JPanel contents = new JPanel();
		contents.setLayout(new BoxLayout(contents, BoxLayout.Y_AXIS));
		//contents.setLayout(new GridLayout(0, 1));
		JPanel typePanel = new JPanel();
		typePanel.add(myTypes);
		JPanel browsePanel = new JPanel();
		browsePanel.setLayout(new BoxLayout(browsePanel, BoxLayout.X_AXIS));
		browsePanel.add(myCommand);
		browsePanel.add(myBrowse);
		JPanel commandPanel = new JPanel();
		//commandPanel.add(browsePanel);
		commandPanel.add(myExplanation);
		browsePanel.setBorder(BorderFactory.createEmptyBorder(0,10,0,10));
		JPanel buttonPanel = new JPanel();
		buttonPanel.add(myOK);
		contents.add(typePanel);
		contents.add(browsePanel);
		contents.add(commandPanel);
		contents.add(buttonPanel);
		setContentPane(contents);
		
		setLocation(300, 300);
		//setSize(260, 160);
		pack();
		setResizable(false);
		setVisible(true);
	}
	
	private class TypeListener implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				myCommand.setText(Settings.getInstance().getProperty((String)e.getItem()));
			}
		}
	}

	private static final long serialVersionUID = 7096036091326818955L;
}
