package org.ilyin.gui.explorerView;

import static java.io.File.separatorChar;
import java.awt.event.*;
import javax.swing.*;
import java.awt.*;
import org.ilyin.model.*;
import org.ilyin.settings.Settings;
import org.ilyin.gui.AbstractView;
import java.util.ArrayList;

public class ExplorerView extends AbstractView {
	private final static String DIR = "directory";
	private final static int HSPACE = 20;
	private final static int VSPACE = 20;
	private final static String UP_ICON_PATH = "/icons/large/up.jpg";
	private final static String UP_TEXT = "..";
	
	private IDirectory myCurrentDir;
	private final ArrayList<UnitImage> myImages;
	private int mySelected;

	private JScrollPane myScrollPane;

	public JScrollPane scrollPane() {
		if (myScrollPane == null) {
			myScrollPane = new JScrollPane(this);
		}
		return myScrollPane;
	}
	
	public ExplorerView(IFileManager manager) {
		myImages = new ArrayList<UnitImage>();
		
		ExplorerViewListener listener = new ExplorerViewListener(); 
		addMouseListener(listener);
		FileManagerHolder.getFileManager().addListener(new ModelListener());

		String path = Settings.getInstance().getProperty(DIR);
		try {
			setDirectory((IDirectory)manager.getUnit(path));	// ???
		} catch(RuntimeException e) {
			setDirectory((IDirectory)manager.getUnit("."));
		}
	}
	
	private void setDirectory(IDirectory dir) {
		//System.err.println("setting current dir: " + dir.getName());
		myCurrentDir = dir;
		Settings.getInstance().setProperty(DIR, myCurrentDir.getCanonicalPath());
		
		myImages.clear();
		UnitImage up = new UnitImage(new ImageIcon(ExplorerView.class.getResource(UP_ICON_PATH)), UP_TEXT);
		mySelected = 0;
		myImages.add(up);
		for (IUnit unit : myCurrentDir.getChildren()) {
			myImages.add(new UnitImage(unit));
		}

		repaint();
	}
	
	protected void paintComponent(Graphics g) {
		g.setColor(Color.WHITE);
		final int width = scrollPane().getWidth();
		g.fillRect(0, 0, width, getHeight());

		int x = HSPACE, y = VSPACE;
		int maxx = 0, maxy = 0;

		int i = 0;
		for (UnitImage im : myImages) {
			im.setX(x);
			im.setY(y);
			if (x + im.getWidth() > maxx)
				maxx = x + im.getWidth();
			if (y + im.getHeight() > maxy)
				maxy = y + im.getHeight();
			/*g.setColor(Color.RED);
			g.fillRect(x, y, im.getWidth(), im.getHeight());
			g.setColor(Color.WHITE);*/
			if (i++ == mySelected) {
				im.draw(g, true); // true!!!
			} else {
				im.draw(g, false);
			}
			x += im.getWidth() + HSPACE;
			if (x + im.getWidth() > width) {
				x = HSPACE;
				y += im.getHeight() + VSPACE;
			}
		}
		
		//System.out.println("width=" + this.getWidth() + "; height=" + this.getHeight());
		//System.out.println("maxx = " + maxx + "; maxy = " + maxy);
		setPreferredSize(new Dimension(maxx + HSPACE, maxy + VSPACE));
	}
	
	public void closing() {
		//System.out.println("ExplorerView.closing()");
	}
	
	public String getViewName() {
		return "explorer style view";
	}
	
	public void left() {
		//System.out.println("left()");
		//if (mySelected > 0)
		mySelected--;
		mySelected += myImages.size();
		mySelected %= myImages.size();
		
		repaint();
	}
	
	public void right() {
		//System.out.println("right()");
		//if (mySelected < myImages.size() - 1)
		mySelected++;
		mySelected %= myImages.size();
		
		repaint();
	}
	
	public void enter() {
		IUnit unit = myImages.get(mySelected).getUnit();
		
		if (unit == null) {
			if (myCurrentDir.getParent() != null)
				setDirectory(myCurrentDir.getParent());							
		} else {
			if (unit instanceof IDirectory) {
				//System.err.println("icon clicked: " + unit.getName());
				setDirectory((IDirectory)unit);
			} else {
				unit.getType().doAction(unit);
			}
		}
	}
	
	public IUnit getSelectedUnit() {
		return myImages.get(mySelected).getUnit();
	}

	private class ExplorerViewListener extends MouseAdapter {
		private void action(IUnit unit) {
			if (unit instanceof IDirectory) {
				//System.err.println("icon clicked: " + unit.getName());
				setDirectory((IDirectory)unit);
			} else {
				unit.getType().doAction(unit);
			}
		}
		
		public void mouseClicked(MouseEvent e) {
			//System.err.println("  mouseClicked(): x=" + e.getX() + "; y=" + e.getY());
			
			// right button click
			if (SwingUtilities.isRightMouseButton(e)) {
				JPopupMenu menu = new JPopupMenu();
				
				for(final UnitImage im : myImages) {
					if (im.contains(e.getX(), e.getY())) {
						JMenuItem rename = new JMenuItem(new AbstractAction() {
							public void actionPerformed(ActionEvent ) {
								String name = JOptionPane.showInputDialog(ExplorerView.this, "Enter new name", "Rename", JOptionPane.QUESTION_MESSAGE);
								if (name != null) {
									//System.out.println("renaming");
									boolean result = FileManagerHolder.getFileManager().rename(im.getUnit(), name);
									if (!result) {
										JOptionPane.showMessageDialog(ExplorerView.this, "Error: can't rename file (directory)!", "", JOptionPane.ERROR_MESSAGE);
									}
								}
							}
							private static final long serialVersionUID = 2634333782751138077L;
						});
						rename.setText("Rename");
						menu.add(rename);

						JMenuItem delete = new JMenuItem(new AbstractAction() {
							public void actionPerformed(ActionEvent arg0) {
								int a = JOptionPane.showConfirmDialog(ExplorerView.this, "Are you sure you want to delete selected item?", "Delete", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
								if (a == JOptionPane.OK_OPTION) {
									boolean result = FileManagerHolder.getFileManager().remove(im.getUnit());
									if (!result) {
										JOptionPane.showMessageDialog(ExplorerView.this, "Error: can't delete file (directory)!", "", JOptionPane.ERROR_MESSAGE);
									}
								}
							}
							private static final long serialVersionUID = 2160391480018271386L;
						});
						delete.setText("Delete");
						menu.add(delete);

						menu.show(ExplorerView.this, e.getX(), e.getY());
						return;
					}
				}

				JMenuItem createDir = new JMenuItem(new AbstractAction() {
					public void actionPerformed(ActionEvent arg0) {
						String name = JOptionPane.showInputDialog(ExplorerView.this, "Enter name", "Create directory", JOptionPane.QUESTION_MESSAGE);
						if (name != null) {
							boolean result = FileManagerHolder.getFileManager().createDir(myCurrentDir, name);
							if (!result) {
								JOptionPane.showMessageDialog(ExplorerView.this, "Error: can't create directory!", "", JOptionPane.ERROR_MESSAGE);
							}
						}
					}
					private static final long serialVersionUID = 393429792451337231L;
				});
				createDir.setText("Create directory");
				menu.add(createDir);
				
				JMenuItem createFile = new JMenuItem(new AbstractAction() {
					public void actionPerformed(ActionEvent arg0) {
						String name = JOptionPane.showInputDialog(ExplorerView.this, "Enter name", "Create file", JOptionPane.QUESTION_MESSAGE);
						if (name != null) {
							boolean result = FileManagerHolder.getFileManager().createFile(myCurrentDir, name);
							if (!result) {
								JOptionPane.showMessageDialog(ExplorerView.this, "Error: can't create file!", "", JOptionPane.ERROR_MESSAGE);
							}
						}
					}
					private static final long serialVersionUID = -6593501072963759083L;
				});
				createFile.setText("Create file");
				menu.add(createFile);
								
				menu.show(ExplorerView.this, e.getX(), e.getY());
				return;
			}
			
			// left button click
			for(final UnitImage im : myImages) {
				if (im.contains(e.getX(), e.getY())) {
					mySelected = myImages.indexOf(im);
					
					repaint();
					
					break;
				}
			}
			
			// double click
			if (e.getClickCount() == 2) {
				//System.out.println("x=" + e.getX() + "; y=" + e.getY());
				for(UnitImage im : myImages) {
					if (im.contains(e.getX(), e.getY())) {
						if (im.getUnit() == null) {
							if (myCurrentDir.getParent() != null)
								setDirectory(myCurrentDir.getParent());							
						} else {
							action(im.getUnit());
						}
						return;
					}
				}
			}
		}
	}
	
	private class ModelListener implements IListener {
		public void fireEvent(IEvent event) {
			if (event.getChangedDirectory().getCanonicalPath().equals(myCurrentDir.getCanonicalPath())) {
				setDirectory(myCurrentDir);
			}
		}
	}
	
	private static final long serialVersionUID = -2128135724681395421L;
}
