package org.ilyin.gui.nortonView;

import javax.swing.*;

import java.awt.event.*;
import java.awt.FontMetrics;
import java.util.List;
import org.ilyin.model.*;
import org.ilyin.settings.Settings;

/**
 * Represents a file list in the GUI.
 * 
 * @author Alexander Ilyin
 */
class FileList extends JPanel {
	private final static String DIR = "directory";

	private IDirectory myCurrentDir;
	private JList myList;
	private JLabel myLabel;
	// width of label in pixels
	private int myWidth = -1;
	
	public FileList(IFileManager manager) {
		myList = new JList();
		myLabel = new JLabel();
		
		String path = Settings.getInstance().getProperty(DIR);
		try {
			setDirectory((IDirectory)manager.getUnit(path));	// ???
		} catch(RuntimeException e) {
			setDirectory((IDirectory)manager.getUnit("."));
		}
		
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		myList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		myList.setCellRenderer(new ImageListCellRenderer());
		
		FileListListener listener = new FileListListener();
		myList.addMouseListener(listener);
		myList.addKeyListener(listener);
		FileManagerHolder.getFileManager().addListener(new ModelListener());
		
		//myLabel.setHorizontalAlignment(JLabel.LEFT);
		//myLabel.setHorizontalTextPosition(JLabel.RIGHT);
		
		add(new JScrollPane(myList));
		add(myLabel);
	}
	
	void setDirectory(IDirectory dir) {
		//System.out.println("setting directory: " + dir.getName());
		myCurrentDir = dir;
		Settings.getInstance().setProperty(DIR, myCurrentDir.getCanonicalPath());

		List<IUnit> children = myCurrentDir.getChildren();
		Object[] data = new Object[children.size() + 1];
		data[0] = "..";
		int i = 1;
		for (IUnit u : children) {
			data[i++] = u;
		}
		
		myList.setListData(data);
		myList.setSelectedIndex(0);

		drawLabel();
	}
	
	void setWidth(int width) {
		myWidth = width;
		//System.out.println("width set: " + width);
		drawLabel();
	}
	
	private void drawLabel() {
		//System.out.println("drawLabel() called");
		myLabel.setText("long long long long long long long long long long long long long long long long long long long long long long long long long string");	// !!!
		if (myWidth == -1)
			return;
		if ((myLabel == null) || (myLabel.getGraphics() == null))
			return;
		//System.out.println("drawing label");

		String path = myCurrentDir.getCanonicalPath();
		//System.out.println("path = " + path);

		final FontMetrics fm = myLabel.getGraphics().getFontMetrics();
		while(path.length() > 0) {
			if (fm.stringWidth(path) <= myWidth)
				break;
			path = path.substring(1);
		}
		//System.out.println("setting: " + path);
		myLabel.setText(path);
		//myLabel.setText(myCurrentDir.getCanonicalPath());
	}
	
	private class FileListListener extends MouseAdapter implements KeyListener {
		private void action(int idx) {
			if (idx == 0) {
				IDirectory p = myCurrentDir.getParent();
				//System.out.println(p.getCanonicalPath());
				if (p != null)
					setDirectory(p);
				return ;
			}
			IUnit unit = myCurrentDir.getChildren().get(idx - 1);
			if (unit.isDirectory()) {
				setDirectory((IDirectory)unit);
			} else {
				unit.getType().doAction(unit);
			}
		}
		
		public void mouseClicked(MouseEvent e) {
			if (e.getClickCount() % 2 == 0) {
				action(myList.locationToIndex(e.getPoint()));
			}
		}

		public void keyPressed(KeyEvent e) {
			boolean changed = false;
			int idx = myList.getSelectedIndex();
			
			if (e.getKeyCode() == KeyEvent.VK_ENTER) {
				//int idx = myList.getSelectedIndex();
				if (idx != -1)
					action(idx);
			} else if (e.getKeyCode() == KeyEvent.VK_F6) {
				//int idx = myList.getSelectedIndex();
				if (idx > 0) {
					IUnit unit = myCurrentDir.getChildren().get(idx - 1);
					String name = JOptionPane.showInputDialog(FileList.this, "Enter new name", "Rename", JOptionPane.QUESTION_MESSAGE);
					if (name != null) {
						 changed = FileManagerHolder.getFileManager().rename(unit, name);
						if (!changed) {
							JOptionPane.showMessageDialog(FileList.this, "Error: can't rename file (directory)!", "", JOptionPane.ERROR_MESSAGE);
						}
					}
				}
			} else if ((e.getKeyCode() == KeyEvent.VK_F8) ||
					   (e.getKeyCode() == KeyEvent.VK_DELETE)) {
				//int idx = myList.getSelectedIndex();
				if (idx > 0) {
					IUnit unit = myCurrentDir.getChildren().get(idx - 1);
					int a = JOptionPane.showConfirmDialog(FileList.this, "Are you sure you want to delete selected item?", "Delete", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
					if (a == JOptionPane.OK_OPTION) {
						changed = FileManagerHolder.getFileManager().remove(unit);
						if (!changed) {
							JOptionPane.showMessageDialog(FileList.this, "Error: can't delete file (directory)!", "", JOptionPane.ERROR_MESSAGE);
						}
					}
				}
			} else if (e.getKeyCode() == KeyEvent.VK_F7) {
				String name = JOptionPane.showInputDialog(FileList.this, "Enter name", "Create directory", JOptionPane.QUESTION_MESSAGE);
				if (name != null) {
					changed = FileManagerHolder.getFileManager().createDir(myCurrentDir, name);
					if (!changed) {
						JOptionPane.showMessageDialog(FileList.this, "Error: can't create directory!", "", JOptionPane.ERROR_MESSAGE);
					}
				}
			} else if ((e.getKeyCode() == KeyEvent.VK_F4) &&
					   (e.isShiftDown())) {
				String name = JOptionPane.showInputDialog(FileList.this, "Enter name", "Create file", JOptionPane.QUESTION_MESSAGE);
				if (name != null) {
					changed = FileManagerHolder.getFileManager().createFile(myCurrentDir, name);
					if (!changed) {
						JOptionPane.showMessageDialog(FileList.this, "Error: can't create file!", "", JOptionPane.ERROR_MESSAGE);
					}
				}
			}
			
			if (changed) {
				//System.out.println(idx);
				//System.out.println("components: " + myList.getComponentCount());
				if (idx < myList.getModel().getSize()) {
					myList.setSelectedIndex(idx);
					//System.out.println("setting index: " + idx);
				} else if (idx == myList.getModel().getSize()) {
					myList.setSelectedIndex(idx - 1);
					//System.out.println("setting index: " + (idx - 1));
				}
			}
		}

		public void keyTyped(KeyEvent e) { }

		public void keyReleased(KeyEvent e) { }
	}
	
	private class ModelListener implements IListener {
		public void fireEvent(IEvent event) {
			if (event.getChangedDirectory().getCanonicalPath().equals(myCurrentDir.getCanonicalPath())) {
				setDirectory(myCurrentDir);
			}
		}
	}
	
	private static final long serialVersionUID = 3934909950071168175L;
}