package org.ilyin.model;

import java.util.List; 

/**
 * Represents a set of methods for filling file system model.
 * 
 * @author Alexander Ilyin
 */
public interface IModelFiller {
	/**
	 * Adds child directory names to the <code>dirs</code> list and
	 * child file names to the <code>files</code> list.
	 * Parent directory is specified by its pathname.
	 * 
	 * @param path directory pathname
	 * @param dirs list to which child directory names to be added
	 * @param files list to which child file names to be added
	 */
	void getChildren(String path, List<String> dirs, List<String> files);

	/**
	 * Returns canonical pathname of the parent directory.
	 * Current directory is specified by its pathname.
	 * 
	 * @param path pathname of the file or directory
	 * @return canonical path of the parent directory, or <code>null</code>
	 * 			if there is no parent directory
	 */
	String getParent(String path);

	/**
	 * Returns canonical patname equivalent to the specified pathname.
	 * 
	 * @param path pathname
	 * @return canonical pathname
	 */
	String getCanonicalPath(String path);
	
	/**
	 * The system-dependent default name-separator character.
	 * 
	 * @return name-separator character
	 */
	char separatorChar();
	
	/**
	 * Reads first characters from a file. 
	 * If there are <code>n</code> characters in it, the length of the
	 * result string will be <code>n</code>. Otherwise, the length will
	 * be equal to the number of characters in the file. 
	 * If I/O error occurs, <code>null</code> is returned.
	 * 
	 * @param path pathname of the file
	 * @param n number of characters to read
	 * @return string containing characters read or <code>null</code>, if failed to read
	 */
	String readChars(String path, int n);
	
	boolean rename(String path, String name);
	
	boolean remove(String path);
	
	boolean createFile(String path);
	
	boolean createDir(String path);
}