package org.ilyin.model.impl;

import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import org.ilyin.model.IDirectory;
import org.ilyin.model.IUnit;

/**
 * The <code>Directory</code> class represents a directory.
 * 
 * @author Alexander Ilyin
 */
class Directory extends Unit implements IDirectory {
	// files and directories this directory contains
	// (this list is sorted and unmodifiable)
	private List<IUnit> myChildren;
	// is myChildren contains correct info
	private boolean myChanged;
	
	/**
	 * Creates a <code>Directory</code> instance with the specified
	 * path. Path may be either absolute and relative.
	 * 
	 * @param path path of the directory
	 */
	public Directory(FileManager manager, String path) {
		super(manager, path);
	}
	
	/**
	 * Creates a <code>Directory</code> instance with the specified
	 * name and parent.
	 * 
	 * @param name name of the directory
	 * @param parent parent directory
	 */
	public Directory(FileManager manager, String name, IDirectory parent) {
		super (manager, name, parent);
	}
	
	public List<IUnit> getChildren() {
		//System.out.println("getChildren() called for " + getName());
		//System.out.println(" myChanged = " + myChanged);
		if ((myChildren == null) || (myChanged)) {
			//System.out.println("children updated");
			myChanged = false;
			myChildren = new ArrayList<IUnit>();

			List<String> dirs = new ArrayList<String>();
			List<String> files = new ArrayList<String>();
			getManager().getFiller().getChildren(getCanonicalPath(), dirs, files);
			for(String name : dirs)
				myChildren.add(new Directory(getManager(), name, this));
			for(String name : files)
				myChildren.add(new File(getManager(), name, this));
			
			Collections.sort(myChildren);
			myChildren = Collections.unmodifiableList(myChildren);
		}
		return myChildren;
	}
	
	public void setChanged() {
		//System.out.println("setChanged() called for " + getName());
		myChanged = true;
	}
	
	public final boolean isDirectory() {
		return true;
	}
	
	public String toString() {
		return "Directory: " + getName();
	}
}
