package org.ilyin.model.impl;

import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import org.ilyin.model.*;

class FileManager implements IFileManager {
	// file types, that system supports
	private List<IFileType> myTypes;
	// model filler
	private IModelFiller myFiller;
	//
	private /*final*/ IDirectory myProjectDir;// = new Directory(this, ".");
	// model listeners
	private List<IListener> myListeners;
	
	public FileManager () {
		myTypes = new ArrayList<IFileType>();
		myListeners = new ArrayList<IListener>();
	}
	
	//  !
	IFileType getType(Unit unit) {
		for (IFileType type : myTypes) {
			if (type.doesBelong(unit))
				return type;
		}

		throw new RuntimeException("Code supposed to be unreachable has been reached.\n" +
								   "No type found for " + unit.getCanonicalPath());
	}
	
	public void setFiller(IModelFiller filler) {
		myFiller = filler;
		myProjectDir = new Directory(this, ".");
	}
	
	public IModelFiller getFiller() {
		return myFiller;
	}

	public IUnit getUnit(String name, IDirectory parent) {
		for(IUnit u : parent.getChildren()) {
			if (u.getName().equals(name)) {
				return u;
			}
		}
		
		throw new RuntimeException("Directory " + name + " doesn't exist.");
	}
	
	public IUnit getUnit(String path) {
		IDirectory current = myProjectDir;
		path = myFiller.getCanonicalPath(path);
		
		while(!path.startsWith(current.getCanonicalPath())) {
			current = current.getParent();
		}

		if (path.equals(current.getCanonicalPath()))
			return current;

		String rem = path.substring(current.getCanonicalPath().length() + 1);
		
		IUnit current2 = current;
		int pos = rem.indexOf(myFiller.separatorChar()); 
		while (pos != -1) {
			String name = rem.substring(0, pos);
			current2 = getUnit(name, (IDirectory)current2);
			
			rem = rem.substring(pos + 1);
			pos = rem.indexOf(myFiller.separatorChar());
		}
		current2 = getUnit(rem, (IDirectory)current2);
		
		return current2;
	}

	public void setType(IFileType type) {
		//type.setAction(action);
		myTypes.add(type);
	}
	
	public List<IFileType> getTypes() {
		return Collections.unmodifiableList(myTypes);
	}

	public void addListener(IListener listener) {
		myListeners.add(listener);
	}

	public void removeListener(IListener listener) {
		myListeners.remove(listener);
	}
	
	public boolean rename(IUnit unit, String name) {
		if (myFiller.rename(unit.getCanonicalPath(), name)) {
			//System.out.println(name + " renamed");
			Directory changed = (Directory)unit.getParent(); 
			changed.setChanged();
			((Unit)unit).setName(name);
			fireEvent(new ModelChangedEvent(changed));			
			return true;
		}

		return false;
	}

	public boolean remove(IUnit unit) {
		if (myFiller.remove(unit.getCanonicalPath())) {
			//System.out.println(unit.getName() + " deleted");
			Directory changed = (Directory)unit.getParent();
			//((Directory)unit.getParent()).setChanged();
			changed.setChanged();
			fireEvent(new ModelChangedEvent(changed));
			return true;
		}
		
		return false;
	}
	
	public boolean createFile(IDirectory parent, String name) {
		// if (parent == null) !!!
		if (myFiller.createFile(parent.getCanonicalPath() + myFiller.separatorChar() + name)) {	// !!!
			//System.out.println(name + " created");
			((Directory)parent).setChanged();
			fireEvent(new ModelChangedEvent(parent));
			return true;
		}
		
		return false;
	}
	
	public boolean createDir(IDirectory parent, String name) {
		if (myFiller.createDir(parent.getCanonicalPath() + myFiller.separatorChar() + name)) {// !!!
			//System.out.println(name + " created");
			((Directory)parent).setChanged();
			fireEvent(new ModelChangedEvent(parent));
			return true;
		}
		
		return false;
	}

	private void fireEvent(IEvent event) {
		for(IListener l : myListeners) {
			l.fireEvent(event);
		}
	}
	
	private class ModelChangedEvent implements IEvent {
		private IDirectory myDir;
		
		public ModelChangedEvent(IDirectory dir) {
			myDir = dir;
		}
		
		public IDirectory getChangedDirectory() {
			return myDir;
		}
	}
}