package org.ilyin.model.impl;

import org.ilyin.model.*;

/**
 * The <code>Unit</code> class represents elements of filesystem 
 * such as files and directories.
 *  
 * @author Alexander Ilyin
 */
abstract class Unit implements IUnit {
	// name of the unit
	private String myName;
	// parent directory
	private IDirectory myParent;
	// absolute and unique path
	private String myCanonicalPath;
	//
	private FileManager myManager;
	// type of the unit
	private IFileType myType;

	/**
	 * Creates a <code>Unit</code> instance with the specified
	 * path. Path may be either absolute and relative.
	 * 
	 * @param path path of the object
	 */
	public Unit(FileManager manager, String path) {
		myManager = manager;
		IModelFiller filler = myManager.getFiller();
		myCanonicalPath  = filler.getCanonicalPath(path);
		//System.out.println(myCanonicalPath);

		int pos = myCanonicalPath.lastIndexOf(filler.separatorChar());
		if (pos == -1) {
			//setName(myCanonicalPath);
			myName = myCanonicalPath;
		} else {
			//setName(myCanonicalPath.substring(pos + 1, myCanonicalPath.length()));
			myName = myCanonicalPath.substring(pos + 1, myCanonicalPath.length());
			//System.out.println(myCanonicalPath.substring(pos + 1, myCanonicalPath.length()));
		}
	}
	
	/**
	 * Creates a <code>Unit</code> instance with the specified
	 * name and parent.  
	 * 
	 * @param name name of the object
	 * @param parent parent directory
	 */
	public Unit(FileManager manager, String name, IDirectory parent) {
		myManager = manager;
		//setName(name);
		myName = name;
		myParent = parent;
		IModelFiller filler = myManager.getFiller();
		myCanonicalPath = filler.getCanonicalPath(
				parent.getCanonicalPath() +
				filler.separatorChar() + myName);
	}
	
	/**
	 * Gets the name of this object.
	 * 
	 * @return name
	 */
	public String getName () {
		return myName;
	}
	
	public void setName(String name) {
		myName = name;
		updatePath();
	}
	
	private void updatePath() {
		// to do!!		
		myCanonicalPath = myCanonicalPath.substring(0, myCanonicalPath.lastIndexOf(myManager.getFiller().separatorChar()) + 1);
		myCanonicalPath += myName;
		// it's necessary to update paths of children here!! 
	}
	
	/**
	 * Gets the parent directory. If it's a root directory, <tt>null</tt>
	 * is returned.
	 * 
	 * @return parent directory or null, if it's root directory
	 */
	public IDirectory getParent () {
		if (myParent == null) {
			String path = myManager.getFiller().getParent(myCanonicalPath);
			if (path != null)
				myParent = new Directory(myManager, path);
		}
		
		return myParent;
	}
	
	public IFileType getType () {
		if (myType == null)
			myType = myManager.getType(this);
		
		return myType;
	}
	
	/**
	 * Gets absolute and unique system-dependent path of this object.
	 * 
	 * @return path
	 */
	public String getCanonicalPath() {
		return myCanonicalPath;
	}
	
	protected FileManager getManager() {
		return myManager;
	}
	
	public int compareTo(IUnit u) {
		if ((isDirectory()) && (!u.isDirectory())) {
			return -1;
		} else if ((!isDirectory()) && (u.isDirectory())) {
			return 1;
		}
		return myName.compareTo(u.getName());
	}
}
