package org.ilyin.settings;

import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import org.w3c.dom.*;
import javax.swing.JOptionPane;
import java.io.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;
import java.util.HashMap;
import static java.io.File.separatorChar;

public class Settings {
	private static final String SETTINGS_FILENAME = System.getProperty("user.home") + separatorChar + "fm_ilyin_settings.xml";
	private static final String ROOT_TAG = "FileManager";
	private static final String OPTION_TAG = "option";
	private static final String PROPERTY_NAME = "name";
	private static final String PROPERTY_VALUE = "value";
	
	private final HashMap<String, String> myProperties;
	
	private static Settings ourInstance;
	
	public static Settings getInstance() {
		if (ourInstance == null)
			ourInstance = new Settings();
		
		return ourInstance;
	}
	
	private Settings() {
		myProperties = new HashMap<String, String>();
		load();
	}
	
	private class SettingsHandler extends DefaultHandler {
		public void startElement(String uri, String localName, String qName, Attributes attributes) {
			if (qName == OPTION_TAG)
				myProperties.put(attributes.getValue(PROPERTY_NAME), attributes.getValue(PROPERTY_VALUE));
		}
	}
	
	private void load() {
		SAXParserFactory factory = SAXParserFactory.newInstance();
		SAXParser myParser = null;
		try {
			myParser = factory.newSAXParser();
		} catch (ParserConfigurationException e) {
			System.out.println("ParserConfigurationException occured!");
			System.out.println(e.getMessage());
			e.printStackTrace();
		} catch (SAXException e) {
			System.out.println("SAXException occured!");
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		
		XMLReader reader;
		//InputStreamReader(InputStream, charset)
		try {//FileInputStream
			reader = myParser.getXMLReader();
			reader.setContentHandler(new SettingsHandler());
			//reader.parse(new InputSource(new FileReader(SETTINGS_FILENAME)));
			reader.parse(new InputSource(new InputStreamReader(new FileInputStream(SETTINGS_FILENAME), "UTF-8")));
		} catch (SAXException e) {
			JOptionPane.showMessageDialog(null, "Error in settings file.",
					  					  "Warning", JOptionPane.WARNING_MESSAGE);
		} catch (FileNotFoundException e) {
			JOptionPane.showMessageDialog(null, "Settings file not found.", 
										  "Warning", JOptionPane.WARNING_MESSAGE);
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, "Can't read settings file.",
					  					  "Warning", JOptionPane.WARNING_MESSAGE);
		}
	}

	public void save() {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        try {
             DocumentBuilder builder = factory.newDocumentBuilder();
             Document doc = builder.newDocument();
             
             Node root = doc.createElement(ROOT_TAG);
             doc.appendChild(root);
             for (String s : myProperties.keySet()) {
            	 Element e = doc.createElement(OPTION_TAG);
            	 e.setAttribute(PROPERTY_NAME, s);
            	 e.setAttribute(PROPERTY_VALUE, myProperties.get(s));
            	 root.appendChild(e);
             }
             
             // creating settings file
             TransformerFactory tf = TransformerFactory.newInstance();
             StreamResult res = new StreamResult(new File(SETTINGS_FILENAME));
             Transformer transformer = tf.newTransformer();
             transformer.setOutputProperty(OutputKeys.INDENT, "yes");
             transformer.transform(new DOMSource(doc), res);
        } catch(ParserConfigurationException e) {
			System.out.println("ParserConfigurationException occured!");
			System.out.println(e.getMessage());
			e.printStackTrace();
        } catch(TransformerConfigurationException e) {
			System.out.println("TransformerConfigurationException occured!");
			System.out.println(e.getMessage());
			e.printStackTrace();
        } catch(TransformerException e) {
			System.out.println("TransformerException occured!");
			System.out.println(e.getMessage());
			e.printStackTrace();
        }
	}

	public String getProperty(String name) {
		return myProperties.get(name);
	}
	
	public void setProperty(String name, String value) {
		myProperties.put(name, value);
	}
}