package org.ilyin.tests;

import org.ilyin.model.IModelFiller;
import java.util.List;
import java.util.ArrayList;

/**
 * A stub for <code>ModelFiller</code> interface.
 * 
 * It has simple implementations to suit testing aims.
 * 
 * @author Alexander Ilyin
 */
public class TestFiller implements IModelFiller {
	private int myGetChildrenCount;
	
	//private String[] childrenDirs = {"catalog", "dir", "test1"};
	private ArrayList<String> myChildrenDirs;
	//private String[] childrenFiles = {"file.txt"};
	private ArrayList<String> myChildrenFiles;
	
	public TestFiller() {
		myChildrenFiles = new ArrayList<String>();
		myChildrenFiles.add("file.txt");
		
		myChildrenDirs = new ArrayList<String>();
		myChildrenDirs.add("catalog");
		myChildrenDirs.add("dir");
		myChildrenDirs.add("test1");
	}
	
	/**
	 * Returns the number of invocations of method <code>getChildren()</code>. 
	 * 
	 * @return number of invocations
	 */
	public int getChildrenInvocationNumber() {
		return myGetChildrenCount;
	}
	
	public void getChildren(String path, List<String> dirs, List<String> files) {
		myGetChildrenCount++;
		
		for (String dir : myChildrenDirs) {
			dirs.add(dir);
		}
		/*dirs.add("catalog");
		dirs.add("dir");
		dirs.add("test1");*/
		
		for (String file : myChildrenFiles) {
			files.add(file);
		}
		//files.add("file.txt");
	}
	
	private int myGetParentCount;
	
	/**
	 * Returns the number of invocations of method <code>getParent()</code>. 
	 * 
	 * @return number of invocations
	 */
	public int getParentInvocationNumber() {
		return myGetParentCount;
	}
	
	public String getParent(String path) {
		myGetParentCount++;

		return "c:" + separatorChar() + "parent";
	}

	public String getCanonicalPath(String path) {
		final String absolutePrefix = "c:" + separatorChar() + "parent";
		
		if (path.startsWith(absolutePrefix))
			return path;
		
		return absolutePrefix + separatorChar() + path;
	}
	
	public char separatorChar() {
		return '\\';
	}
	
	private int myReadCharsCount;
	
	public int getReadCharsInvocationNumber() {
		return myReadCharsCount;
	}
	
	public String readChars(String path, int n) {
		myReadCharsCount++;
		if (n < 0)
			throw new IllegalArgumentException("Trying ro read " + n + 
											   " characters from " + path);

		final String str = "abcde";
		
		if (n >= str.length())
			return str;
		
		return str.substring(0, n);
	}

	public boolean rename(String path, String newName) {
		String oldName = path.substring(path.lastIndexOf(separatorChar()) + 1);
		
		/*for (int i = 0; i < childrenDirs.length; i++) {
			if (childrenDirs[i].equals(oldName)) {
				childrenDirs[i] = newName;
				return true;
			}
		}*/
		int i = 0;
		for(String str : myChildrenDirs) {
			if (str.equals(oldName)) {
				myChildrenDirs.set(i, newName);
				return true;
			}
			i++;
		}
		/*for (int i = 0; i < childrenFiles.length; i++) {
			if (childrenFiles[i].equals(oldName)) {
				childrenFiles[i] = newName;
				return true;
			}
		}*/
		i = 0;
		for (String str : myChildrenFiles) {
			if (str.equals(oldName)) {
				myChildrenFiles.set(i, newName);
				return true;
			}
			i++;
		}
		
		return true;
	}

	public boolean remove(String path) {
		String name;
		if (path.lastIndexOf(separatorChar()) == -1) {
			name = path;
		} else {
			name = path.substring(path.lastIndexOf(separatorChar()) + 1);
		}

		if (myChildrenFiles.contains(name)) {
			myChildrenFiles.remove(name);
			return true;
		}

		if (myChildrenDirs.contains(name)) {
			myChildrenDirs.remove(name);
			return true;
		}
		
		return false;
	}
	
	public boolean createFile(String path) {
		//throw new RuntimeException("Not implemented");
		String name;
		if (path.lastIndexOf(separatorChar()) == -1) {
			name = path;
		} else {
			name = path.substring(path.lastIndexOf(separatorChar()) + 1);
		}
		//System.out.println("FILE NAME=" + name);
		myChildrenFiles.add(name);
		
		return true;
	}
	
	public boolean createDir(String path) {
		//throw new RuntimeException("Not implemented");
		String name;
		if (path.lastIndexOf(separatorChar()) == -1) {
			name = path;
		} else {
			name = path.substring(path.lastIndexOf(separatorChar()) + 1);
		}
		//System.out.println("DIR NAME=" + name);
		myChildrenDirs.add(name);
		
		return true;
	}
}