package org.ilyin.tests;

import java.io.File;
import java.io.FilenameFilter;
import java.util.List;
import java.util.ArrayList;

class TestLoader extends ClassLoader {
	public ITest[] loadTests(String dirName) {
		File[] files = new File(dirName).listFiles(new ClassnameFilter());
		List<ITest> tests = new ArrayList<ITest>();
				
		for (File file : files) {
			Class c = null;
			try {
				c = loadClass(dirName.replace(File.separatorChar, '.') + '.' + 
							  file.getName().substring(0, file.getName().lastIndexOf('.')));
			} catch (ClassNotFoundException e) {
				System.out.println("Class not found:");
				System.out.println(e.getMessage());
			}
			
			try {
				if (ITest.class.isAssignableFrom(c))
					tests.add((ITest)c.newInstance());
			} catch(IllegalAccessException e) {
				System.out.println("IllegalAccessException occured:");
				System.out.println(e.getMessage());				
			} catch(InstantiationException e) {
				System.out.println("InstantiationException occured:");
				System.out.println(e.getMessage());
			}
		}
		
		ITest[] result = new ITest[tests.size()];
		return tests.toArray(result);
	}
	
	private static class ClassnameFilter implements FilenameFilter {
		public boolean accept(File dir, String name) {
			return name.endsWith(".class");
		}
	}
}
